function ssh2Mozilla(observer) {
  inherit(this, new baseProtocol());
  this.observer = observer;

  setTimeout(this.keepAlive.bind(this), 60000);
}

ssh2Mozilla.prototype = {
  // override base class variables
  protocol : 'ssh2',

  transport : null,
  client : null,
  shell : null,
  privatekey   : "",                                                             // private key for ssh connections
  width  : 80,
  height : 24,
  refreshRate : 10,
  bufferOut : '',


  connect : function(reconnect) {
    this.setupConnect(reconnect);
    this.observer.version = this.version;

    try {                                                                        // create a control socket
      var proxyInfo = null;
      var self      = this;

      if (this.proxyType != "") {                                                // use a proxy
        proxyInfo = this.proxyService.newProxyInfo(this.proxyType, this.proxyHost, this.proxyPort, 0, 30, null);
      }

      this.controlTransport = this.transportService.createTransport(null, 0, this.host, parseInt(this.port), proxyInfo);

      var controlOutstream  = this.controlTransport.openOutputStream(0, 0, 0);
      var controlInstream   = this.controlTransport.openInputStream(0, 0, 0);
      this.controlInstream  = Components.classes["@mozilla.org/binaryinputstream;1"].createInstance(Components.interfaces.nsIBinaryInputStream);
      this.controlInstream.setInputStream(controlInstream);

      this.controlOutstream  = Components.classes["@mozilla.org/binaryoutputstream;1"].createInstance(Components.interfaces.nsIBinaryOutputStream);
      this.controlOutstream.setOutputStream(controlOutstream);

      var dataListener = {                                                       // async data listener for the control socket
        onStartRequest  : function(request, context) { },

        onStopRequest   : function(request, context, status) {
          self.legitClose = self.client.legitClose;
          self.onDisconnect();
        },

        onDataAvailable : function(request, context, inputStream, offset, count) {
          try {
            self.transport.fullBuffer += self.controlInstream.readBytes(count);  // read data

            if (!this.gotWelcomeMessage && self.transport.fullBuffer.indexOf('\n') == self.transport.fullBuffer.length - 1) {
              self.onConnected();
            }

            self.transport.run();
          } catch(ex) {
            self.observer.onDebug(ex);

            if (ex instanceof paramikojs.ssh_exception.AuthenticationException) {
              self.client.legitClose = true;
              self.loginDenied(ex.message);
              return;
            }

            self.onDisconnect();
          }
        }
      };

      var pump = Components.classes["@mozilla.org/network/input-stream-pump;1"].createInstance(Components.interfaces.nsIInputStreamPump);
      pump.init(controlInstream, -1, -1, 0, 0, false);
      pump.asyncRead(dataListener, null);

      var shell_success = function(shell) {
        self.shell = shell;

        self.loginAccepted();
        self.isReady = true;
        self.input();
      };

      this.client = new paramikojs.SSHClient();
      this.client.set_missing_host_key_policy(new paramikojs.AskPolicy());
      var file = Components.classes["@mozilla.org/file/directory_service;1"].createInstance(Components.interfaces.nsIProperties)
                                     .get("ProfD", Components.interfaces.nsILocalFile);
      file.append("known_hosts");
      var host_keys = !localFile.init('~/.ssh/known_hosts') && sys.platform == 'win32' ? file.path : '~/.ssh/known_hosts';
      if (sys.platform != 'win32' && !localFile.init('~/.ssh').exists()) {
        var dir  = localFile.init('~/.ssh');

        try {
          dir.create(Components.interfaces.nsILocalFile.DIRECTORY_TYPE, 0700);
          localFile.overrideOSXQuarantine(dir.path);
        } catch(ex) {
          debug(ex);
          error(gStrbundle.getString("dirFail"));
        }
      }
      this.client.load_host_keys(host_keys);

      var auth_success = function() {
        self.client.invoke_shell('xterm', self.width, self.height, shell_success);
      };

      this.transport = this.client.connect(this.observer, this.controlOutstream, auth_success,
                                      this.host, parseInt(this.port), this.login, this.password, null, this.privatekey);

    } catch(ex) {
      this.observer.onDebug(ex);
      this.onDisconnect();
    }
  },

  cleanup : function(isAbort) {
    this._cleanup();
  },

  sendQuitCommand : function(legitClose) {                                       // called when shutting down the connection
    this.client.close(legitClose);
    this.kill();
  },

  keepAlive              : function() {
    if (this.isConnected && this.keepAliveMode) {
      this.client._transport.global_request('keepalive@lag.net', null, false);
    }

    setTimeout(this.keepAlive.bind(this), 60000);
  },

  input : function() {    
    try {
      var stdin = this.shell.recv(65536);
    } catch(ex if ex instanceof paramikojs.ssh_exception.WaitException) {
      this.check_stderr();
      return;
    }
    if (stdin) {
      this.observer.onStdin(stdin, 'input', 'input');
    }
    this.check_stderr();
  },

  check_stderr : function() {
    try {
      var stderr = this.shell.recv_stderr(65536);
    } catch(ex if ex instanceof paramikojs.ssh_exception.WaitException) {
      setTimeout(this.input.bind(this), this.refreshRate);
      return;
    }
    if (stderr) {
      this.observer.onError(stderr, 'error', 'error');
    }

    setTimeout(this.input.bind(this), this.refreshRate);
  },

  output : function(out) {
    this.bufferOut += out;
    this.send_output();
  },

  send_output : function() {
    while (this.bufferOut.length > 0) {
      try {
        var n = this.shell.send(this.bufferOut);
      } catch(ex if ex instanceof paramikojs.ssh_exception.WaitException) {
        var self = this;
        var wait_callback = function() {
          self.send_output();
        }
        setTimeout(wait_callback, 10);
        return;
      }
      if (n <= 0) { // eof
        break;
      }
      this.bufferOut = this.bufferOut.substring(n);
    }
    //setTimeout(this.send_output.bind(this), this.refreshRate);
  },

  recoverFromDisaster    : function() { /* do nothing */ }
}
